const nodeExternals = require('webpack-node-externals');
const { merge } = require('webpack-merge');
const { getProjectPath, deleteFolders } = require('../common/utils');
const baseConfig = require('./webpack.base');

module.exports = function (appConfig, extendEnv, { isServer, isHotServer, isProd }) {
  const result = merge(
    baseConfig(appConfig, extendEnv, {
      isServer,
      isHotServer,
      isProd
    }),
    {
      target: 'node',
      entry: getProjectPath('/src/serverEntry/index.tsx'),
      output: {
        path: getProjectPath('/build'),
        publicPath: '/',
        filename: 'server-bundle.js',
        libraryTarget: 'commonjs2',
        chunkFilename: 'js/[name].[chunkhash].js'
      },
      module: {
        rules: [
          {
            oneOf: [
              {
                test: /\.(scss|css|sass)$/,
                use: [
                  {
                    loader: require.resolve('ignore-loader')
                  }
                ]
              }
            ]
          }
        ]
      },
      // 服务端打包的时候忽略外部的npm包
      externals: nodeExternals({
        // 当然外部的css还是可以打进来的
        allowlist: /\.(css|less|scss)$/
      })
    }
  );

  const buildServerOutput = getProjectPath(result.output.path);
  deleteFolders(buildServerOutput);

  return result;
};
